@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2021-2022 Mike Fährmann
+# Copyright 2021-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -80,7 +80,7 @@ class EromeExtractor(Extractor):
         for params["page"] in itertools.count(1):
             page = self.request(url, params=params).text
 
-            album_ids = EromeAlbumExtractor.pattern.findall(page)
+            album_ids = EromeAlbumExtractor.pattern.findall(page)[::2]
             yield from album_ids
 
             if len(album_ids) < 36:
