@@ -12,7 +12,6 @@ from .common import Extractor, Message
 from .. import text, util, exception
 from ..cache import cache
 import itertools
-import json
 
 BASE_PATTERN = r"(?:https?://)?(?:\w+\.)?pinterest\.[\w.]+"
 
@@ -113,7 +112,7 @@ class PinterestExtractor(Extractor):
 class PinterestPinExtractor(PinterestExtractor):
     """Extractor for images from a single pin from pinterest.com"""
     subcategory = "pin"
-    pattern = BASE_PATTERN + r"/pin/([^/?#&]+)(?!.*#related$)"
+    pattern = BASE_PATTERN + r"/pin/([^/?#]+)(?!.*#related$)"
     test = (
         ("https://www.pinterest.com/pin/858146903966145189/", {
             "url": "afb3c26719e3a530bb0e871c480882a801a4e8a5",
@@ -122,7 +121,7 @@ class PinterestPinExtractor(PinterestExtractor):
         }),
         # video pin (#1189)
         ("https://www.pinterest.com/pin/422564377542934214/", {
-            "pattern": r"https://v\.pinimg\.com/videos/mc/hls/d7/22/ff"
+            "pattern": r"https://v\d*\.pinimg\.com/videos/mc/hls/d7/22/ff"
                        r"/d722ff00ab2352981b89974b37909de8.m3u8",
         }),
         ("https://www.pinterest.com/pin/858146903966145188/", {
@@ -148,8 +147,8 @@ class PinterestBoardExtractor(PinterestExtractor):
     subcategory = "board"
     directory_fmt = ("{category}", "{board[owner][username]}", "{board[name]}")
     archive_fmt = "{board[id]}_{id}"
-    pattern = (BASE_PATTERN + r"/(?!pin/)([^/?#&]+)"
-               "/(?!_saved|_created|pins/)([^/?#&]+)/?$")
+    pattern = (BASE_PATTERN + r"/(?!pin/)([^/?#]+)"
+               "/(?!_saved|_created|pins/)([^/?#]+)/?$")
     test = (
         ("https://www.pinterest.com/g1952849/test-/", {
             "pattern": r"https://i\.pinimg\.com/originals/",
@@ -199,7 +198,7 @@ class PinterestBoardExtractor(PinterestExtractor):
 class PinterestUserExtractor(PinterestExtractor):
     """Extractor for a user's boards"""
     subcategory = "user"
-    pattern = BASE_PATTERN + r"/(?!pin/)([^/?#&]+)(?:/_saved)?/?$"
+    pattern = BASE_PATTERN + r"/(?!pin/)([^/?#]+)(?:/_saved)?/?$"
     test = (
         ("https://www.pinterest.com/g1952849/", {
             "pattern": PinterestBoardExtractor.pattern,
@@ -224,7 +223,7 @@ class PinterestAllpinsExtractor(PinterestExtractor):
     """Extractor for a user's 'All Pins' feed"""
     subcategory = "allpins"
     directory_fmt = ("{category}", "{user}")
-    pattern = BASE_PATTERN + r"/(?!pin/)([^/?#&]+)/pins/?$"
+    pattern = BASE_PATTERN + r"/(?!pin/)([^/?#]+)/pins/?$"
     test = ("https://www.pinterest.com/g1952849/pins/", {
         "pattern": r"https://i\.pinimg\.com/originals/[0-9a-f]{2}"
                    r"/[0-9a-f]{2}/[0-9a-f]{2}/[0-9a-f]{32}\.\w{3}",
@@ -246,10 +245,10 @@ class PinterestCreatedExtractor(PinterestExtractor):
     """Extractor for a user's created pins"""
     subcategory = "created"
     directory_fmt = ("{category}", "{user}")
-    pattern = BASE_PATTERN + r"/(?!pin/)([^/?#&]+)/_created/?$"
+    pattern = BASE_PATTERN + r"/(?!pin/)([^/?#]+)/_created/?$"
     test = ("https://www.pinterest.de/digitalmomblog/_created/", {
         "pattern": r"https://i\.pinimg\.com/originals/[0-9a-f]{2}"
-                   r"/[0-9a-f]{2}/[0-9a-f]{2}/[0-9a-f]{32}\.jpg",
+                   r"/[0-9a-f]{2}/[0-9a-f]{2}/[0-9a-f]{32}\.(jpg|png)",
         "count": 10,
         "range": "1-10",
     })
@@ -271,7 +270,7 @@ class PinterestSectionExtractor(PinterestExtractor):
     directory_fmt = ("{category}", "{board[owner][username]}",
                      "{board[name]}", "{section[title]}")
     archive_fmt = "{board[id]}_{id}"
-    pattern = BASE_PATTERN + r"/(?!pin/)([^/?#&]+)/([^/?#&]+)/([^/?#&]+)"
+    pattern = BASE_PATTERN + r"/(?!pin/)([^/?#]+)/([^/?#]+)/([^/?#]+)"
     test = ("https://www.pinterest.com/g1952849/stuff/section", {
         "count": 2,
     })
@@ -309,7 +308,7 @@ class PinterestSearchExtractor(PinterestExtractor):
 
     def __init__(self, match):
         PinterestExtractor.__init__(self, match)
-        self.search = match.group(1)
+        self.search = text.unquote(match.group(1))
 
     def metadata(self):
         return {"search": self.search}
@@ -322,7 +321,7 @@ class PinterestRelatedPinExtractor(PinterestPinExtractor):
     """Extractor for related pins of another pin from pinterest.com"""
     subcategory = "related-pin"
     directory_fmt = ("{category}", "related {original_pin[id]}")
-    pattern = BASE_PATTERN + r"/pin/([^/?#&]+).*#related$"
+    pattern = BASE_PATTERN + r"/pin/([^/?#]+).*#related$"
     test = ("https://www.pinterest.com/pin/858146903966145189/#related", {
         "range": "31-70",
         "count": 40,
@@ -341,7 +340,7 @@ class PinterestRelatedBoardExtractor(PinterestBoardExtractor):
     subcategory = "related-board"
     directory_fmt = ("{category}", "{board[owner][username]}",
                      "{board[name]}", "related")
-    pattern = BASE_PATTERN + r"/(?!pin/)([^/?#&]+)/([^/?#&]+)/?#related$"
+    pattern = BASE_PATTERN + r"/(?!pin/)([^/?#]+)/([^/?#]+)/?#related$"
     test = ("https://www.pinterest.com/g1952849/test-/#related", {
         "range": "31-70",
         "count": 40,
@@ -349,13 +348,13 @@ class PinterestRelatedBoardExtractor(PinterestBoardExtractor):
     })
 
     def pins(self):
-        return self.api.board_related(self.board["id"])
+        return self.api.board_content_recommendation(self.board["id"])
 
 
 class PinterestPinitExtractor(PinterestExtractor):
     """Extractor for images from a pin.it URL"""
     subcategory = "pinit"
-    pattern = r"(?:https?://)?pin\.it/([^/?#&]+)"
+    pattern = r"(?:https?://)?pin\.it/([^/?#]+)"
 
     test = (
         ("https://pin.it/Hvt8hgT", {
@@ -371,7 +370,7 @@ class PinterestPinitExtractor(PinterestExtractor):
         self.shortened_id = match.group(1)
 
     def items(self):
-        url = "https://api.pinterest.com/url_shortener/{}/redirect".format(
+        url = "https://api.pinterest.com/url_shortener/{}/redirect/".format(
             self.shortened_id)
         response = self.request(url, method="HEAD", allow_redirects=False)
         location = response.headers.get("Location")
@@ -459,10 +458,10 @@ class PinterestAPI():
         options = {"section_id": section_id}
         return self._pagination("BoardSectionPins", options)
 
-    def board_related(self, board_id):
+    def board_content_recommendation(self, board_id):
         """Yield related pins of a specific board"""
-        options = {"board_id": board_id, "add_vase": True}
-        return self._pagination("BoardRelatedPixieFeed", options)
+        options = {"id": board_id, "type": "board", "add_vase": True}
+        return self._pagination("BoardContentRecommendation", options)
 
     def user_pins(self, user):
         """Yield all pins from 'user'"""
@@ -504,7 +503,10 @@ class PinterestAPI():
             "username_or_email": username,
             "password"         : password,
         }
-        data = {"data": json.dumps({"options": options}), "source_url": ""}
+        data = {
+            "data"      : util.json_dumps({"options": options}),
+            "source_url": "",
+        }
 
         try:
             response = self.extractor.request(
@@ -523,7 +525,10 @@ class PinterestAPI():
 
     def _call(self, resource, options):
         url = "{}/resource/{}Resource/get/".format(self.root, resource)
-        params = {"data": json.dumps({"options": options}), "source_url": ""}
+        params = {
+            "data"      : util.json_dumps({"options": options}),
+            "source_url": "",
+        }
 
         response = self.extractor.request(
             url, params=params, headers=self.headers,
